(* file: eratosthenes.ml
 * author: Bob Muller
 *
 * CS1101 Computer Science I
 *
 * More work with lists. Computing the prime numbers less than or
 * equal to N.  First a naive and inefficient version.
 *)
let isFactor m n = n mod m = 0
let iSqrt i = int_of_float (sqrt (float_of_int i))

(* isPrime : int -> bool            from earlier
 *)
let isPrime n =
  let top = iSqrt n in
  let rec repeat m =
    match m > top with
    | true -> true
    | false -> (match isFactor m n with
                | true  -> false
                | false -> repeat (m + 1))
  in
  repeat 2

(* primes : int -> int list
 *)
let rec primes n =
  match n < 2 with
  | true  -> []
  | false -> (match isPrime n with
              | true -> n :: (primes (n - 1))
              | false -> primes (n - 1))

               
(* Now for the more efficient version of Eratosthenes. 
 *)
let range lo hi =
  let rec loop i acc =
    match i = hi with
    | true -> acc
    | false -> loop (i + 1) (i :: acc)
  in
  List.rev (loop lo [])

let rec allButMultiples n ns =
  match ns with
  | [] -> []
  | m :: ms -> match isFactor n m with
               | true -> allButMultiples n ms
               | false -> m :: (allButMultiples n ms)

(* eratosthenes : int -> int list
 *)
let eratosthenes n =
  let ns = range 2 (n + 1) in
  let rec sieve ms primes =
    match ms with
    | [] -> primes
    | x :: xs -> let xs' = allButMultiples x xs
                 in
                 sieve xs' (x :: primes)
  in
  sieve ns []        
